import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class companionGame extends PApplet {

//soi.benjamin@soih
PShape p1, p2;
Player player1, player2;
ArrayList<Bullet> bullets;
ArrayList<Missile> missiles;
ArrayList<Asteroid> asteroids;

public void setup() {
  background(0);
  player1 = new Player(0, true); player2 = new Player(PI, false);
  
  
  p1 = loadShape("bodybazookadown.svg");
  p2 = loadShape("bodygunup.svg");

  keys = new boolean[8];
  for (int i = 0; i < keys.length; ++i) keys[i] = false;
  bullets = new ArrayList<Bullet>();
  asteroids = new ArrayList<Asteroid>();
  missiles = new ArrayList<Missile>();
}
class Bullet {
  float x, y, angle, velocity;
  Bullet(float _x, float _y, float _angle, float _velocity) {
    x = _x; 
    y = _y; 
    angle = _angle; 
    velocity = _velocity;
  }
  public void display() {
    pushMatrix();
    translate(x, y);
    noStroke();
    ellipse(0, 0, 3, 3);
    popMatrix();
  }
  public void mechanics() {
    x += sin (angle) * velocity;
    y += cos (angle) * velocity;
  }
}

class Missile {
  float x, y, angle, velocity;
  Missile(float _x, float _y, float _angle, float _velocity) {
    x = _x; 
    y = _y; 
    angle = _angle; 
    velocity = _velocity;
  }
  public void display() {
    pushMatrix();
    translate(x, y);
    noStroke();
    fill(255);
    rect(0, 0, 13, 13);
    popMatrix();
  } 
  public void mechanics() {
    float targetX = -1000, targetY = -1000;
    int closestTargetId = -1;
    for (int i = 0; i < asteroids.size(); ++i) {
      if (dist(asteroids.get(i).x, asteroids.get(i).y, x, y) < dist(targetX, targetY, x, y)) {
        targetX = asteroids.get(i).x;
        targetY = asteroids.get(i).y;
        closestTargetId = i;
      }
    }
    if (closestTargetId != -1) {
      /*targetX = 1000;
      targetY = -1000;*/
      float newAng = atan2(y - targetY, x - targetX);
      newAng += PI;
      if(newAng >= angle) angle += 0.1f;
      else angle -= 0.1f;
      angle %= 2 * PI;
    }
    x += cos (angle) * velocity;
    y += sin (angle) * velocity;
  }
}
public void draw() {
  background(0);
  environment();
  hitboxes();
  displaySpaceShip();
  player1.mechanics();
  player1.display();
  player2.mechanics();
  player2.display();
  for(int i = 0; i < bullets.size(); ++i) {
    bullets.get(i).display();
    bullets.get(i).mechanics();
    if(bullets.get(i).x < -1000 || bullets.get(i).y < -1000 || bullets.get(i).x > width + 2000 || bullets.get(i).y > height + 2000) bullets.remove(i);
  }
  for(int i = 0; i < missiles.size(); ++i) {
    missiles.get(i).display();
    missiles.get(i).mechanics();
    if(missiles.get(i).x < -1000 || missiles.get(i).y < -1000 || missiles.get(i).x > width + 2000 || missiles.get(i).y > height + 2000) missiles.remove(i);
  }
  for(int i = 0; i < asteroids.size(); ++i) {
    asteroids.get(i).display();
    asteroids.get(i).mechanics();
    if(asteroids.get(i).x < -2000 || asteroids.get(i).y < -2000 || asteroids.get(i).x > width + 2000 || asteroids.get(i).y > height + 2000) asteroids.remove(i);
  }
}
class Player {
  float angle;
  boolean dead, upOrDown;
  Player(float _angle, boolean _upOrDown) {
    angle = _angle;
    dead = false;
    upOrDown = _upOrDown;
  }
  public void display() {
    if (!dead) {
      angle %= (2 * PI);
      pushMatrix();
      translate(width/2, height/2);
      rotate(angle);
      fill(255, 0, 0);
      rectMode(CENTER);
      //rect(SHIPRADIUS/2, 0, 30, 30);
      pushMatrix();
      int shapeX = 75;
      translate(SHIPRADIUS/2 - shapeX/2,0);
      rotate(-PI/2);
  shapeMode(CENTER);

      if(upOrDown) shape(p1, 0,0, shapeX, shapeX);
      else shape(p2, 0, 0, shapeX, shapeX);
      popMatrix();
      popMatrix();
    }
  }

  public void mechanics() {
    if (!dead) {
      if (upOrDown) {
        if (keys[1]) angle += 0.05f;
        if (keys[3]) angle -= 0.05f;
      } else {
        if (keys[5]) angle += 0.05f;
        if (keys[6]) angle -= 0.05f;
      }
    }
  }
  public float getRealPositionX() {
    float x = cos(angle) * SHIPRADIUS/2.4f + width / 2;
    float y = sin(angle) * SHIPRADIUS/2.4f + height / 2;
    fill(255);
    ellipse(x, y, 100, 100);
    return x;
  }
    public float getRealPositionY() {
    float x = cos(angle) * SHIPRADIUS/2.4f + width / 2;
    float y = sin(angle) * SHIPRADIUS/2.4f + height / 2;
        fill(255);

    ellipse(x, y, 100, 100);
    return x;
  }
};
class Asteroid {
  float x, y, angle, velocity, health;
  Asteroid (int _x, int _y, float _angle, float _velocity) {
    x = _x; y = _y; angle = _angle; velocity = _velocity;
    health = 5;
  }
  public void display() {
    fill(255, (255.f/5.f) * health, (255.f/5.f) * health);
    noStroke();
    ellipse(x, y, 50, 50);
  }
  
  public void mechanics() {
    x += sin (angle) * velocity;
    y += cos (angle) * velocity;
  }
}
public void environment () {
  if(PApplet.parseInt(random(0, 5)) == 1) {
      int x = PApplet.parseInt(random(-1000, width + 1000)), y = PApplet.parseInt(random(-1000, height + 1000));
      if(! (x >= -100 && x <= width + 100 && y >= -100 && y <= height + 100) ) {
        Asteroid a = new Asteroid(x, y, random(0, 2 * PI), 2);
        asteroids.add(a);
    }
  }

}
int SHIPRADIUS = 1;
public void displaySpaceShip() {
  stroke(255);
  strokeWeight(10);
  fill(0);
  SHIPRADIUS = min((int)(PApplet.parseFloat(width) / 3),(int)( PApplet.parseFloat(height) / 3));
  ellipse(width/2, height/2, SHIPRADIUS, SHIPRADIUS);
}
public void hitboxes() {
  float bulletRadius = 2.5f;
  float asteroidRadius = 50 - 10;
  int playerRadius = 100;
  ArrayList<Integer> asteroidsToRemove = new ArrayList<Integer> ();
  ArrayList<Integer> bulletsToRemove = new ArrayList<Integer> ();

  for (int i = 0; i < bullets.size(); ++i) {
    float bx = bullets.get(i).x, by = bullets.get(i).y;

    for (int j = 0; j < asteroids.size(); ++j) {
      if (dist(bx, by, asteroids.get(j).x, asteroids.get(j).y) < bulletRadius + asteroidRadius) {

        //bulletsToRemove.add(i);

        bullets.remove(i);
        asteroids.get(j).health--;
        //
        //asteroidsToRemove.add(j);
        if (asteroids.get(j).health <= 0) {
          for (int k = 0; k < 4; ++k) {
            Bullet b = new Bullet(asteroids.get(j).x, asteroids.get(j).y, random(0, PI), 3);
            bullets.add(b);
          }
          asteroids.remove(j);
        }

        j = asteroids.size();
      }
    }
  }

  for (int i = 0; i < missiles.size(); ++i) {
    float bx = missiles.get(i).x, by = missiles.get(i).y;

    for (int j = 0; j < asteroids.size(); ++j) {
      if (dist(bx, by, asteroids.get(j).x, asteroids.get(j).y) < bulletRadius + asteroidRadius) {
        //bulletsToRemove.add(i);
        //asteroidsToRemove.add(j);
        for (int k = 0; k < 4; ++k) {
          Bullet b = new Bullet(asteroids.get(j).x, asteroids.get(j).y, random(0, PI), 3);
          bullets.add(b);
        }
        missiles.remove(i);
        asteroids.remove(j);

        j = asteroids.size();
      }
    }
  }

  for (int i = 0; i < asteroids.size(); ++i) {
    float player1X = player1.getRealPositionX();
    float player1Y = player1.getRealPositionY();
    float player2X = player2.getRealPositionX();
    float player2Y = player2.getRealPositionY();

    if (player1.dead == false && dist(asteroids.get(i).x, asteroids.get(i).y, player1X, player1Y) < playerRadius) {
      player1.dead = true;
      asteroids.remove(i);
      ++i;
      for (int k = 0; k < 20; ++k) {
        Bullet b = new Bullet(player1X, player1Y, random(0, 2 * PI), 3);
        bullets.add(b);
      }
    }
    if (player2.dead == false && dist(asteroids.get(i).x, asteroids.get(i).y, player2X, player2Y ) < playerRadius) {
      player2.dead = true;
      asteroids.remove(i);
      ++i;
      for (int k = 0; k < 20; ++k) {
        Bullet b = new Bullet(player2X, player2Y, random(0, 2 * PI), 3);
        bullets.add(b);
      }
    }
  }

  //for(int a : asteroidsToRemove) asteroids.remove(a);
  //for(int a : bulletsToRemove) bullets.remove(a);
}

boolean[] keys;
public void keyPressed() {
  if (key == CODED) {
    if (keyCode == UP) keys[4] = true;
    
    if (keyCode == LEFT) keys[5] = true;
    if (keyCode == RIGHT) keys[6] = true;
    if (keyCode == DOWN) keys[7] = true;
    if(keys[4] == true) { 
      Bullet b = new Bullet(cos(player2.angle + 0.1f) * (SHIPRADIUS/2 - 75) + width / 2, sin(player2.angle + 0.1f) * (SHIPRADIUS/2 - 75) + height / 2, -player2.angle - PI/2 % (2 * PI), 10);
      if(player2.dead == false) bullets.add(b);
    }
  } else {
    if (key == 'w' || key == 'W') keys[0] = true;
    if (key == 'a' || key == 'A') keys[1] = true;
    if (key == 's' || key == 'S') keys[2] = true;
    if (key == 'd' || key == 'D') keys[3] = true;
  }
}

public void keyReleased() {
  if (key == CODED) {
    if (keyCode == UP) keys[4] = false;
    if (keyCode == LEFT) keys[5] = false;
    if (keyCode == RIGHT) keys[6] = false;
    if (keyCode == DOWN) keys[7] = false;
  } else {
    if (key == 'w' || key == 'W') {
      keys[0] = false;
      if(player1.dead == false) {Missile b = new Missile(cos(player1.angle - 0.20f) * (SHIPRADIUS/2 - 75.f/3.5f) + width / 2, sin(player1.angle - 0.20f) * (SHIPRADIUS/2 - 75.f/3.5f) + height / 2, -player1.angle + PI/2 % (2 * PI), 2);
      missiles.add(b);}
    }
    if (key == 'a' || key == 'A') keys[1] = false;
    if (key == 's' || key == 'S') {
      keys[2] = false;
      /*Bullet b = new Bullet(cos(player1.angle) * (SHIPRADIUS/2) + width / 2, sin(player1.angle) * (SHIPRADIUS/2) + height / 2, -player1.angle + PI + PI/2 % (2 * PI), 10);  
      bullets.add(b);*/
    }
    if (key == 'd' || key == 'D') keys[3] = false;
  }
}
  public void settings() {  size(1024, 768); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "companionGame" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
